#! /bin/sh

# This is the initialisation script for Syllable Desktop.
# 
# ************************************************************************** #
#                            DO NOT EDIT THIS FILE!                          #
# ************************************************************************** #
#
# init.sh is called directly by the init process; do not call this file
# yourself.
# This script in turn calls /etc/profile, user-early-init.sh, network-init.sh
# and user-init.sh  If you wish to modify your configuration or initialisation
# process you should edit these files instead.

# Log local kernel output into /var/log/kernel
/system/bin/dbterm 8 >> /var/log/kernel &

# Add a few additional symlinks which simplify navigation around the
# filesystem
ln -s usr /resources
ln -s boot/Applications /Applications
ln -s boot/documentation /documentation
#ln -s home/root /root

# Configure basic environment.
# BASh runs this, too, but do it earlier for the benefit of processes started
# here.
source /etc/profile

# Silently empty the temporary directory
if [ "$TEMP" != '' ]
then
	rm -r $TEMP/* > /dev/null 2>&1
fi

# If the graphical login isn't available, failsafe to a standalone server and
# abort further initialisation
if [ ! -e /system/bin/dlogin ]
then
	aterm &
	exit 1
fi

# Start the graphical login and servers
/system/bin/dlogin < /dev/null >> /var/log/desktop 2>&1 &

# No more environment variables can be defined for the user's environment
# after DLogin has started

/system/mediaserver > /dev/null &
/system/registrar > /dev/null &

# Check for network changes & configure the network if required
/Applications/Preferences/Network --detect

source /system/network-init.sh

# inetd is a special case and is always started first if INetUtils is
# installed.

if [ -e /resources/indexes/libexec/inetd ]
then
	/resources/indexes/libexec/inetd &
fi

# Packages that require initalisation can include early-init and init directories,
# which should contain the init script(s). E.g. Apache would have init/apache which
# would call apachectl, OpenSSH would have init/sshd which would start
# sshd etc.
# The package manager will collect all of these scripts together in
# /resources/indexes/early-init/ and /resources/indexes/init/; all we need to do is
# run each script in turn.

# Run the late init scripts

for pkg_init in `ls /system/indexes/init/`
do
	source /system/indexes/init/$pkg_init
done

for pkg_init in `ls /resources/indexes/init/`
do
	source /resources/indexes/init/$pkg_init
done

# Do user initialisation and exit
source /system/user-init.sh

exit 0
